/// @file       declare.h
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $ khv $
/// @date       $ 2014-10-13 $
/// @brief      File contains wave file format declarations.

#ifndef _DECLARE_H_
#define _DECLARE_H_

/// Macros.
#define RIFF_ID					0x46464952		///< "RIFF" Little Endian format.
#define WAVE_ID					0x45564157		///< "WAVE" Little Endian format.
#define FMT_CHUNK_ID			0x20746d66		///< "fmt " Little Endian format.
#define DATA_CHUNK_ID			0x61746164		///< "data" Little Endian format.
#define ID_LENGTH						 4		///< Total bytes in Id.
#define PCM_FORMAT_CHUNK_SIZE			16		///< Without extra format bytes.
#define MIN_VOLUME				0x00000000		///< Minimum volume.
#define MAX_VOLUME				0xFFFFFFFF		///< Maximum volume.

/// RIFF file header.
typedef struct tRiffFileHeader
{
	DWORD   riffHeader;
	DWORD   riffHeaderSize;
	DWORD   waveType;
}riffFileHeader;

/// RIFF format chunk header.
typedef struct tRiffFormatChunkHeader
{
	DWORD   formatChunkHeader;   
	DWORD   formatChunkSize;
}riffFormatChunkHeader;

/// RIFF data chunk header.
typedef struct tRiffDataChunkHeader
{
	DWORD   dataChunkHeader;   
	DWORD   dataChunkSize;
}riffDataChunkHeader;

/// Current state. 
typedef enum tState
{
	UNDEFINED,
	PAUSED,
	STOPPED,
	PLAYING
}playOutState;

/// Operation to perform.
typedef enum tOperation
{
	PLAY = 1,
	PAUSE,
	STOP,
	VOLUME_UP,
	VOLUME_DOWN,
	EXIT
}operation;
#endif