/// Header files.
#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>
#include "declare.h"

/// Procedures declarations.
void Cleanup(FILE *filePtr, playOutState state);

/// Static declarations.
static WAVEHDR waveHeader = {0};
static HWAVEOUT waveOut = {0};

int wmain()
{
	CHAR *fileName = "\\Program Files\\vcpp_audio_playback_waf\\Sleep_Away.wav";
	FILE *fPlay = NULL;
	errno_t error;
	WAVEFORMATEX formatChunk = {0};
	DWORD select = 0;
	playOutState currentState = UNDEFINED;
	DWORD volume = MAX_VOLUME;
	BOOL loopTerminate = FALSE;
	MMRESULT retVal = MMSYSERR_NOERROR;
	riffFileHeader fileInfo = {0};
	riffFormatChunkHeader formatChunkInfo = {0};
	riffDataChunkHeader dataChunkInfo = {0};

	/// Get number of waveform output devices present in the system.
	if (waveOutGetNumDevs() == 0)
	{
		printf("\nNo waveform output device present in the system for playback.");
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Opens the file to play.
	error = fopen_s(&fPlay, fileName, "rb");
	if (error != 0)
	{
		printf("Unable to open the file.");
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Reads file type and format.
	if (fread((VOID*)&fileInfo, sizeof(fileInfo), 1, fPlay) == 1)
	{
		if (fileInfo.riffHeader != RIFF_ID || fileInfo.waveType != WAVE_ID)
		{
			printf("\nFile type not supported.");
			/// Clean-up.
			Cleanup(fPlay, currentState);
			fflush(stdin);
			getchar();
			return 0;
		}
	}
	else
	{
		printf("\nUnable to read the file.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Reads format id ("fmt ") and size of format chunk.
	if (fread((VOID*)&formatChunkInfo, sizeof(formatChunkInfo), 1, fPlay) == 1)
	{
		if (formatChunkInfo.formatChunkHeader != FMT_CHUNK_ID)
		{
			printf("\nFile type not supported.");
			/// Clean-up.
			Cleanup(fPlay, currentState);
			fflush(stdin);
			getchar();
			return 0;
		}
	}
	else
	{
		printf("\nUnable to read the file.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Reads audio format.
	if (fread((VOID*)&formatChunk, PCM_FORMAT_CHUNK_SIZE, 1, fPlay) == 1)
	{
		if (fseek(fPlay, formatChunkInfo.formatChunkSize - PCM_FORMAT_CHUNK_SIZE, SEEK_CUR))
		{
			printf("\nUnable to read the file.");
			/// Clean-up.
			Cleanup(fPlay, currentState);
			fflush(stdin);
			getchar();
			return 0;
		}
	}
	else
	{
		printf("\nUnable to read the file.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Searches for "data" id in the file.
	while (loopTerminate == FALSE)
	{
		if (fread((VOID*)&dataChunkInfo, sizeof(dataChunkInfo), 1, fPlay) == 1)
		{
			if (dataChunkInfo.dataChunkHeader == DATA_CHUNK_ID)
			{
				 break;
			}
			if (fseek(fPlay, dataChunkInfo.dataChunkSize, SEEK_CUR))
			{
				loopTerminate = TRUE;
			}
		}
		else
		{
			loopTerminate = TRUE;
		}
	}
	if (loopTerminate == TRUE)
	{
		printf("\nFile corrupted.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	waveHeader.dwBufferLength = dataChunkInfo.dataChunkSize;
	/// Allocates memory for the buffer to play.
	waveHeader.lpData = (CHAR*)malloc(dataChunkInfo.dataChunkSize);
	if (waveHeader.lpData == NULL)
	{
		printf("\nMemory not enough.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Reads the data to play.
	if (fread((VOID*)waveHeader.lpData, sizeof(CHAR), dataChunkInfo.dataChunkSize, fPlay) != dataChunkInfo.dataChunkSize)
	{
		printf("\nUnable to read the file.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Closes the file.
	fclose(fPlay);
	/// Opens a waveform output device for playback.
	retVal = waveOutOpen(&waveOut, WAVE_MAPPER, &formatChunk, 0, 0, CALLBACK_NULL);
	if (retVal != MMSYSERR_NOERROR)
	{
		if (retVal == WAVERR_BADFORMAT)
		{
			printf("\nFile format is not supported by playback device.");
		}
		else
		{
			printf("\nUnable to open the device for playback.");
		}
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}
	/// Prepares a waveform data block for playback.
	retVal = waveOutPrepareHeader(waveOut, &waveHeader, sizeof(waveHeader));
	if (retVal != MMSYSERR_NOERROR)
	{
		printf("\nUnable to prepare the data to play.");
		/// Clean-up.
		Cleanup(fPlay, currentState);
		fflush(stdin);
		getchar();
		return 0;
	}

	while (select != EXIT)
	{
	printf("\n\n********************** Menu **********************");
	printf("\n1. Play \n2. Pause \n3. Stop \n4. Volume Up \n5. Volume Down \n6. Exit\t ");
	fflush(stdin);
	scanf_s("%d", &select);
	switch(select)
	{
	case PLAY:
		{
			if (currentState == PLAYING && !(waveHeader.dwFlags & WHDR_DONE))
			{
				printf("\nAlready in play mode.");
				break;
			}
			else if (currentState == PAUSED)
			{
				waveOutRestart(waveOut);
			}
			else
			{
				waveHeader.dwFlags &= 0x0E;
				/// Sends a data block to the specified waveform output device.
				waveOutWrite(waveOut, &waveHeader, sizeof(waveHeader));
				/// Sets the volume of a waveform output device.
				waveOutSetVolume(waveOut, volume);
			}
			currentState = PLAYING;
			printf("\nPlaying track...");
			break;
		}
	case PAUSE:
		{
			if (currentState != PLAYING)
			{
				printf("\nFirst play the track.");
			}
			else
			{
				/// Pauses playback on a specified waveform output device.
				waveOutPause(waveOut);
				currentState = PAUSED;
				printf("\nPaused.");
			}
			break;
		}
	case STOP:
		{
			if (currentState != PLAYING)
			{
				printf("\nFirst play the track.");
			}
			else
			{
				/// Stops playback on a specified waveform output device.
				waveOutReset(waveOut);
				waveHeader.dwFlags &= 0x0E;
				currentState = STOPPED;
				printf("\nStopped.");
			}
			break;
		}
	case VOLUME_UP:
		{
			/// Queries the current volume setting of a waveform output device.
			waveOutGetVolume(waveOut, &volume);
			if (volume == MAX_VOLUME)
			{
				printf("\nMaximum volume.");
			}
			else
			{
				if (volume > MAX_VOLUME - 0x10001000)
				{
					volume = MAX_VOLUME;
				}
				else
				{
					volume += 0x10001000;
				}
				/// Sets the volume of a waveform output device.
				waveOutSetVolume(waveOut, volume);
				printf("\nVolume increased.");
			}
			break;
		}
	case VOLUME_DOWN:
		{
			/// Queries the current volume setting of a waveform output device.
			waveOutGetVolume(waveOut, &volume);
			if (volume == MIN_VOLUME)
			{
				printf("\nMinimum volume.");
			}
			else
			{
				if (volume < 0x10001000)
				{
					volume = MIN_VOLUME;
				}
				else
				{
					volume -= 0x10001000;
				}
				/// Sets the volume of a waveform output device.
				waveOutSetVolume(waveOut, volume);
				printf("\nVolume decreased.");
			}
			break;
		}
	case EXIT:
		{
			break;
		}
	default:
		{
			printf("\nInvalid selection. Try again.");
		}
	}
	}
	/// Clean-up.
	Cleanup(fPlay, currentState);
	return 0;
}

/// Clean-up.
void Cleanup(FILE *filePtr, playOutState state)
{
	/// Closes the file, if open.
	if (filePtr != NULL)
	{
		fclose(filePtr);
	}
	/// Stops the playback, if running or paused.
	if (state == PLAYING || state == PAUSED)
	{
		/// Stops playback on a specified waveform output device.
		waveOutReset(waveOut);
	}
	/// Un-prepares the header, if prepared for output.
	if ((waveHeader.dwFlags & WHDR_PREPARED) || waveHeader.reserved)
	{
		waveOutUnprepareHeader(waveOut, &waveHeader, sizeof(waveHeader));
	}
	/// Closes the waveform-audio output device, if open.
	if (waveOut != NULL)
	{
		waveOutClose(waveOut);
	}
	/// Frees the memory, if allocated.
	if (waveHeader.lpData != NULL)
	{
		free(waveHeader.lpData);
	}
}	